/* Copyright  2007 by Christian Fuchsberger and Lukas Forer info@pedvizapi.org.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License <http://www.pedvizapi.org/gpl.txt>
 * for more details. 
 */

package core;

import java.awt.Color;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.DefaultTableModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import pedviz.algorithms.*;
import pedviz.algorithms.sugiyama.RandomSplitter;
import pedviz.algorithms.sugiyama.Splitter;
import pedviz.algorithms.sugiyama.SugiyamaLayout;
import pedviz.graph.*;
import pedviz.loader.*;
import pedviz.view.*;
import pedviz.view.rules.*;
import pedviz.view.symbols.SymbolHaplotypes;
import pedviz.view.symbols.SymbolSexFemale;
import pedviz.view.symbols.SymbolSexMale;
import pedviz.view.symbols.SymbolSexUndesignated;
import pedviz.view.symbols3d.SymbolSexFemale3d;
import pedviz.view.symbols3d.SymbolSexFemaleLow;
import pedviz.view.symbols3d.SymbolSexMale3d;
import pedviz.view.symbols3d.SymbolSexMaleLow;
import pedviz.view.symbols3d.SymbolSexUndesignated3d;
import pedviz.view.symbols3d.SymbolSexUndesignatedLow;
import view.AppView;
import view.dialogs.*;
import view.tools.*;

import core.tools.*;
import pedviz.haplotype.*;

/**
 * AppController
 * 
 * @author Luki
 * 
 */
public class AppController implements NodeListener {

	private static final int VIEW2D = 1;

	private static final int VIEW3D = 2;

	private AppView view;

	private DefaultNodeView defaultNodeView;

	private DefaultEdgeView defaultEdgeView;

	private int highlighting = Highlighter.MATERNAL;

	private boolean synchHighlighter = false;

	private Splitter splitter = new RandomSplitter();

	private Vector<GraphView> graphviews;

	private HashMap<String, Vector<String>> lookUpTable;

	private HashMap<String, Vector<String>> lookUpTableDouble;

	private Vector<String> traits, doubleTraits;

	private Graph cgraph = null;

	private ToolView tool;

	private boolean useQuestions = false;

	private Vector<Graph> graphs;

	private Haplotypes haplotypes;

	private DialogTools toolsDialog;

	HaplotypesView haploView;
	
	private String currentDir = "";

	public AppController() {

		graphviews = new Vector<GraphView>();
		graphs = new Vector<Graph>();

		lookUpTable = new HashMap<String, Vector<String>>();
		lookUpTableDouble = new HashMap<String, Vector<String>>();
		doubleTraits = new Vector<String>();

		defaultNodeView = new DefaultNodeView();
		defaultNodeView.setColor(Color.DARK_GRAY);
		defaultNodeView.addHintAttribute("pid");

		defaultEdgeView = new DefaultEdgeView();
		defaultEdgeView.setHighlightedColor(Color.red);

	}

	private GraphView createGraphview(int mode) {

		GraphView graphview = null;
		if (mode == VIEW2D) {
			graphview = new GraphView2D();
		} else {
			graphview = new GraphView3D();
		}
		graphview.setSelectionEnabled(true);
		graphview.setMultiselection(true);
		graphview.addNodeListener(this);
		graphview.setBackgroundColor(Color.white);

		if (Application.getInstance().isJ3dInstalled() && mode == VIEW3D) {
			GraphView3D graphview3D = (GraphView3D) graphview;
			boolean flag = !Application.getInstance().isSet("no_antialiasing");
			graphview3D.setAntialiasingEnabled(flag);
			flag = !Application.getInstance().isSet("no_transparency");
			graphview3D.setTransparencyEnabled(flag);
			flag = !Application.getInstance().isSet("no_light");
			graphview3D.setLightEnabled(flag);
			flag = !Application.getInstance().isSet("low_quality");
			graphview3D.setMaterialEnabled(flag);
		}

		return graphview;
	}

	/**
	 * Sets the View Object
	 * 
	 * @param view
	 */
	public void setView(AppView view) {
		this.view = view;

		String mode = Application.getInstance()
				.getProperty("pedvizapi.ui.view");
		if (mode != null) {
			if (mode.contains("2D") || mode.contains("2d")) {
				GraphView graphView2d = createGraphview(VIEW2D);
				view.addGraphview(graphView2d);
				graphviews.add(graphView2d);
			}

			if (mode.contains("3D") || mode.contains("3d")) {
				if (Application.getInstance().isJ3dInstalled()) {
					GraphView graphView3d = createGraphview(VIEW3D);
					view.addGraphview(graphView3d);
					graphviews.add(graphView3d);
				}
			}
		} else {
			GraphView graphView2d = createGraphview(VIEW2D);
			view.addGraphview(graphView2d);
			graphviews.add(graphView2d);

			if (Application.getInstance().isJ3dInstalled()) {
				GraphView graphView3d = createGraphview(VIEW3D);
				view.addGraphview(graphView3d);
				graphviews.add(graphView3d);
			}

		}
		toolsDialog = new DialogTools(view);
		registerTools();
		view.restoreWorkspace();
		view.setFamiliesEnabled(false);
		view.setControlsEnabled(false);
		view.setActionsEnabled(false);
		view.setUseQuestions(false);
	}

	/**
	 * Registers a Tool
	 * 
	 * @param tool
	 */
	private void registerTool(ToolView tool) {
		view.addTool(tool);
	}

	private void registerTool(ToolView tool, boolean extern) {
		view.addTool(tool, extern);
	}

	private void registerToolInDialog(ToolView tool) {
		toolsDialog.addTool(tool);
	}

	/**
	 * Registers all Tools
	 * 
	 */
	private void registerTools() {
		registerTool(new TraitsView(new TraitsController()));
		registerTool(new QualitativeTraitsView(
				new QualitativeTraitsController()));
		// registerTool(new FilterView(new FilterController()));
		registerTool(new FilterView(new FilterController()));
		registerTool(new HighlightingView(new HighlightingController()));
		registerToolInDialog(new GenotypesView(new GenotypesController()));
		haploView = new HaplotypesView(new HaplotypesController());
		registerToolInDialog(haploView);
		haploView.setEnabled(false);

		if (Application.getInstance().isJ3dInstalled()) {
			registerTool(new LayoutView(new LayoutController()));
		}
		// tool = new QuestionnaireView(new QuestionnaireController());
		// registerTool(tool, true);
		registerTool(null);
	}

	/**
	 * Loads the Graph from File
	 * 
	 */
	public void loadFromFile() {
		JFileChooser dialog = new JFileChooser();
		dialog.addChoosableFileFilter(new CsvFilter());
		dialog.setMultiSelectionEnabled(false);
		dialog.showOpenDialog(view);
		if (dialog.getSelectedFile() != null) {
			String filename = dialog.getSelectedFile().getPath();
			
			CsvGraphLoader loader = createGraphLoader(filename);
			DialogOpen dialogOpen = new DialogOpen(view);
			dialogOpen.setAllowImportTraits(false);
			dialogOpen.setColData(loader.getColumns());
			setTraits(loader.getColumns());
			if (dialogOpen.execute()) {
				currentDir = dialog.getSelectedFile().getParent();
				view.setWaitCursor();
				Graph graph = new Graph();
				loader.setSettings(dialogOpen.getId(), dialogOpen.getMom(),
						dialogOpen.getDad());
				loader.load(graph);
				graph.setName(filename);

				loadCompleteGraph(graph);

				if (dialogOpen.checkErrors()) {
					ErrorChecking errors = new ErrorChecking(graphs, dialogOpen
							.getSexMale(), dialogOpen.getSexFemale());
					errors.run();
					if (errors.getErrors().size() > 0) {
						DialogErrors errDialog = new DialogErrors(view);
						errDialog.setErrors(errors.getErrors());
						if (errDialog.execute()) {
							Vector<String> log = GraphRepair
									.insertMissingParents(graphs, dialogOpen
											.getSexMale(), dialogOpen
											.getSexFemale());
							String temp = "";
							for (String message : log) {
								temp += message + "\n";
							}
							JOptionPane.showMessageDialog(null, temp);
							for (Graph g : graphs) {
								g.buildHierarchie(new HierarchieUpDown());
							}
						}
					}
				}

				buildRules(dialogOpen.getSex(), dialogOpen.getSexMale(),
						dialogOpen.getSexFemale());
				view.setDefaultCursor();
			}

		}
	}


	private void buildRules(String sex, String maleChar, String femaleChar) {
		for (GraphView graphview : graphviews) {
			if (Application.getInstance().isJ3dInstalled()) {
				if (graphview instanceof GraphView3D) {
					if (Application.getInstance().isSet("low_quality")) {
						graphview.addRule(new ShapeRule(sex, femaleChar,
								new SymbolSexFemaleLow()));
						graphview.addRule(new ShapeRule(sex, maleChar,
								new SymbolSexMaleLow()));
						graphview.addRule(new ShapeRule(sex, "-1",
								new SymbolSexUndesignatedLow()));
						graphview.addRule(new ColorRule("virtual", "1",
								Color.LIGHT_GRAY));
					} else {
						SymbolSexFemale3d female = new SymbolSexFemale3d();
						female.setStyle(SymbolSexFemale3d.STYLE_TURM);
						graphview
								.addRule(new ShapeRule(sex, femaleChar, female));
						SymbolSexMale3d male = new SymbolSexMale3d();
						male.setStyle(SymbolSexMale3d.STYLE_TURM);
						graphview.addRule(new ShapeRule(sex, maleChar, male));
						SymbolSexUndesignated3d other = new SymbolSexUndesignated3d();
						other.setStyle(SymbolSexMale3d.STYLE_TURM);
						graphview.addRule(new ShapeRule(sex, "-1", other));
						graphview.addRule(new ColorRule("virtual", "1",
								Color.LIGHT_GRAY));
					}
				}
			}
			if (graphview instanceof GraphView2D) {
				graphview.addRule(new ShapeRule(sex, maleChar,
						new SymbolSexMale()));
				graphview.addRule(new ShapeRule(sex, femaleChar,
						new SymbolSexFemale()));
				graphview.addRule(new ShapeRule(sex, "-1",
						new SymbolSexUndesignated()));
				graphview.addRule(new ColorRule("virtual", "1",
						Color.LIGHT_GRAY));
			}
		}

	}

	public void showTool() {
		tool.showAsFrame();
	}

	/**
	 * Creates a CsvGraphLoad object
	 * 
	 * @param filename
	 * @return
	 */
	private CsvGraphLoader createGraphLoader(String filename) {
		CsvGraphLoader loader = new CsvGraphLoader(filename, ",");
		if (loader.getColumns().size() == 1) {
			loader = new CsvGraphLoader(filename, ";");
			if (loader.getColumns().size() == 1) {
				loader = new CsvGraphLoader(filename, " ");
				while (loader.getColumns().size() == 1) {
					String splitter = JOptionPane
							.showInputDialog("Please enter the split-charakter:");
					if (splitter != null) {
						loader = new CsvGraphLoader(filename, splitter);
					} else {
						return null;
					}
				}
			}
		}
		return loader;
	}

	/**
	 * Loads the complete Graph
	 * 
	 * @param graph
	 */
	private void loadCompleteGraph(Graph graph) {

		DialogProgress dialog = new DialogProgress(view, "Load Pedigree...");

		FamilySplitter splitter = new FamilySplitter(graph);
		dialog.addTask(splitter);
		dialog.run();
		String name = graph.getName();
		view.setPedigreeName(name);
		updateLookUpTables(graph);
		graph.clear();

		DefaultTreeModel model = view.getFamiliesModel();
		DefaultMutableTreeNode root = new DefaultMutableTreeNode();
		root.setUserObject(name);
		model.setRoot(root);

		ArrayList<Graph> families = splitter.getFamilies();
		graphs.clear();
		graphs.addAll(families);
		for (int i = 0; i < families.size(); i++) {
			Graph family = families.get(i);
			 if (family.getAllNodes().size() >= 5) {

			SameParents sameParents = new SameParents(family);
			sameParents.run();

			family.buildHierarchie(new HierarchieUpDown());
			family.setName("Family_" + i + " (" + family.getSize() + ")");

			DefaultMutableTreeNode child = new DefaultMutableTreeNode();
			child.setUserObject(family);
			root.add(child);
			 }
		}

		for (ToolView tool : view.getTools()) {
			tool.getController().afterGraphLoaded(tool, graph);
		}

		for (ToolView tool : toolsDialog.getTools()) {
			tool.getController().afterGraphLoaded(tool, graph);
		}

		view.setFamiliesEnabled(true);
		view.setControlsEnabled(false);
		view.setActionsEnabled(false);
		view.setUseQuestions(true);

		haplotypes = null;
		haploView.setEnabled(false);
		haploView.setTraits(null);

		model.reload();

		this.cgraph = null;
	}

	public void importHaplotypes() {
		JFileChooser dialog = new JFileChooser();
		dialog.addChoosableFileFilter(new MerlinChrFilter());
		dialog.setMultiSelectionEnabled(false);
		dialog.showOpenDialog(view);
		if (dialog.getSelectedFile() != null) {
			String filename = dialog.getSelectedFile().getPath();
			JFileChooser dialog2 = new JFileChooser();
			dialog2.addChoosableFileFilter(new MapFilter());
			dialog2.setMultiSelectionEnabled(false);
			dialog2.showOpenDialog(view);
			if (dialog2.getSelectedFile() != null) {
				String mapfile = dialog2.getSelectedFile().getPath();

				haplotypes = new MerlinHaplotypes(filename, mapfile);
				haploView.setEnabled(true);
				haploView.setTraits(haplotypes.getMarkers());
				// displayGraph(cgraph);
			}
		}
	}

	/**
	 * Displays a Graph object
	 * 
	 * @param object
	 */
	public void updatePedigree(Object object) {
		if (object instanceof Graph) {
			Graph graph = (Graph) object;
			if (getGraph() != graph) {
				displayGraph(graph);
				centerGraph();
				this.cgraph = graph;
				for (ToolView tool : view.getTools()) {
					tool.getController().afterGraphSelected(tool, graph);
				}
				for (ToolView tool : toolsDialog.getTools()) {
					tool.getController().afterGraphSelected(tool, graph);
				}
			}
		}
	}

	/**
	 * Displays a Graph object
	 * 
	 * @param graph
	 */
	private void displayGraph(Graph graph) {

		for (GraphView graphview : graphviews) {
			int mode = graphview instanceof GraphView2D ? VIEW2D : VIEW3D;
			graphview.setGraph(layout(graph, mode));
		}
		view.setActionsEnabled(true);
		view.setControlsEnabled(true);
		view.setUseQuestions(useQuestions);
		view.setSplitEnabled(false);
	}

	/**
	 * Updates Details on Demand
	 * 
	 * @param node
	 */
	public void updateDetails(Node node) {
		String[][] data = new String[traits.size()][2];
		for (int i = 0; i < traits.size(); i++) {
			data[i][0] = traits.get(i);
			if (node.getUserData(traits.get(i)) != null) {
				data[i][1] = node.getUserData(traits.get(i)).toString();
			} else {
				data[i][1] = "-";
			}
		}
		DefaultTableModel model = view.getDetailsModel();
		model.setDataVector(data, new String[] { "Trait", "Value" });
	}

	public void showSettingsDialog() {
		if (toolsDialog.execute()) {
			for (ToolView tool : toolsDialog.getTools()) {
				tool.getController().updatePressed(tool);
			}
		}
	}

	/**
	 * NodeEvents
	 */
	public void onNodeEvent(NodeEvent event) {
		switch (event.getType()) {
		case NodeEvent.MOUSE_ENTER:
			updateDetails(event.getNode());

			if (highlighting != -1) {
				ArrayList<Node> nodes = Highlighter.findLineOfDescents(
						getGraph(), event.getNode(), highlighting);

				if (synchHighlighter) {
					for (GraphView graphview : graphviews) {
						graphview.unHighlightAll();
						graphview.highlight(nodes);
					}
				} else {
					event.getGraphView().unHighlightAll();
					event.getGraphView().highlight(nodes);
				}
			}
			break;
		case NodeEvent.MOUSE_LEAVE:
			if (synchHighlighter) {
				for (GraphView graphview : graphviews) {
					graphview.unHighlightAll();
				}
			} else {
				event.getGraphView().unHighlightAll();
			}
			break;

		case NodeEvent.SELECTED:
			for (GraphView graphview : graphviews) {
				graphview.select(event.getNode());
				// in graphview2d aufnehmen
				if (graphview instanceof GraphView2D) {
					graphview.updateGraphView();
				}
			}
			view.setSplitEnabled(graphviews.get(0).getSelectionCount() > 0);
			break;
		case NodeEvent.DESELECTED:
			for (GraphView graphview : graphviews) {
				graphview.deselect(event.getNode());
				// in graphview2d aufnehmen
				if (graphview instanceof GraphView2D) {
					graphview.updateGraphView();
				}
			}
			view.setSplitEnabled(graphviews.get(0).getSelectionCount() > 0);
			break;
		case NodeEvent.ALL_DESELECTED:
			for (GraphView graphview : graphviews) {
				graphview.deselect();
				if (graphview instanceof GraphView2D) {
					graphview.updateGraphView();
				}
			}
			view.setSplitEnabled(false);
			break;
		}
	}

	/**
	 * Exports Graph as CSV
	 * 
	 */
	public void exportAsCsv() {
		JFileChooser dialog = new JFileChooser();
		dialog.addChoosableFileFilter(new CsvFilter());
		dialog.setMultiSelectionEnabled(false);
		dialog.showSaveDialog(view);
		if (dialog.getSelectedFile() != null) {
			String filename = dialog.getSelectedFile().getPath();
			view.setWaitCursor();
			getGraph().exportAsCsvFile(filename);
			view.setDefaultCursor();
			JOptionPane.showMessageDialog(view, "Wrote File: " + filename,
					"Export CSV", JOptionPane.INFORMATION_MESSAGE);
		}
	}

	/**
	 * Export the Viz as JPEG
	 * 
	 */
	public void exportAsJpeg() {
		DialogExport dialog = new DialogExport(view);
		dialog.setFilename(currentDir +"\\" + cgraph.getName()+".jpg");
		Vector<String> views = new Vector<String>();
		for (GraphView g: graphviews){
			views.add(g instanceof GraphView2D ? "View 2d" : "View 2,5d");
		}
		dialog.setViews(views);
		if (dialog.execute()){
			view.setWaitCursor();
			graphviews.get(dialog.getIndex()).exportJPEG(dialog.getFilename(), dialog.getScale(), dialog.isGrayScale());
			view.setDefaultCursor();
			JOptionPane.showMessageDialog(view, "Wrote File: " + dialog.getFilename(),
					"Export JPEG", JOptionPane.INFORMATION_MESSAGE)	;		
		}
	}

	/**
	 * Creates the layout for a Graph object.
	 * 
	 * @param graph
	 * @param mode
	 *            AppView.VIEW_3D for 3d-Layout, AppView.VIEW_2D for 2d-Layout
	 * @return
	 */
	private LayoutedGraph layout(Graph graph, int mode) {
		view.setWaitCursor();
		DialogProgress dialog = new DialogProgress(view, "Layout Pedigree...");
		SugiyamaLayout layout = new SugiyamaLayout(graph, defaultNodeView,
				defaultEdgeView);
		if (mode == VIEW3D && splitter != null) {
			layout.setSplitter(splitter);
		}
		dialog.addTask(layout);
		RubberBands rubberBands = new RubberBands(layout.getLayoutGraph());
		rubberBands.setDepth(550f);
		if (mode == VIEW2D)
			rubberBands.setHorizontalGap(3);
		rubberBands.setVerticalSpacing(80f);
		dialog.addTask(rubberBands);
		dialog.run();
		view.setDefaultCursor();
		return layout.getLayoutGraph();
	}

	/**
	 * Extracts selection
	 * 
	 */
	public void splitSelection() {
		if (graphviews.get(0).getSelectionCount() > 1) {
			String splitname = JOptionPane
					.showInputDialog("Please enter a name for this part of the pedigree");
			if (splitname != null) {
				Cluster cluster = new Cluster();
				for (Node node : graphviews.get(0).getSelection()) {
					cluster.addNode(node);
				}
				Graph newGraph = new Graph(cluster);
				newGraph.setName(splitname);
				newGraph.buildHierarchie(new HierarchieUpDown());
				DefaultMutableTreeNode node = new DefaultMutableTreeNode(
						newGraph);
				view.getSelectedNode().add(node);
				for (GraphView graphview : graphviews) {
					graphview.deselect();
					// in graphview2d aufnehmen
					if (graphview instanceof GraphView2D) {
						graphview.updateGraphView();
					}
				}
				view.getFamiliesModel().reload();
				view.setSplitEnabled(graphviews.get(0).getSelectionCount() > 0);
			}
		}
	}

	/**
	 * Clears selection
	 * 
	 */
	public void clearSelection() {
		for (GraphView graphview : graphviews) {
			graphview.deselect();
		}
		view.setSplitEnabled(false);
	}

	/**
	 * Returns all GraphView objects
	 * 
	 * @return
	 */
	public Vector<GraphView> getGraphViews() {
		return graphviews;
	}

	/**
	 * Returns View
	 * 
	 * @return
	 */
	public Frame getView() {
		return view;
	}

	/**
	 * Exit
	 * 
	 */
	public void exit() {
		System.exit(0);
	}

	/**
	 * Turns on/off rotating on X-Axis
	 * 
	 * @param e
	 */
	public void rotateX(ActionEvent e) {
		JCheckBoxMenuItem s = (JCheckBoxMenuItem) e.getSource();
		for (GraphView graphview : graphviews) {
			if (graphview instanceof GraphView3D) {
				((GraphView3D) graphview).setXRotatingEnabled(s.isSelected());
			}
		}
	}

	/**
	 * Turns on/off rotating on Y-Axis
	 * 
	 * @param e
	 */
	public void rotateY(ActionEvent e) {
		JCheckBoxMenuItem s = (JCheckBoxMenuItem) e.getSource();
		for (GraphView graphview : graphviews) {
			if (graphview instanceof GraphView3D) {
				((GraphView3D) graphview).setYRotatingEnabled(s.isSelected());
			}
		}
	}

	/**
	 * Shows About-Dialog
	 * 
	 */
	public void showAboutDialog() {
		new DialogAbout(view).execute();
	}

	/**
	 * Turns on/off edges
	 * 
	 */
	public void showEdges() {
		for (GraphView graphview : graphviews) {
			boolean visible = graphview.isEdgeVisible();
			graphview.setEdgeVisible(!visible);
		}
	}

	/**
	 * Centers the Graph
	 * 
	 */
	public void centerGraph() {
		view.resetSliders();
		for (GraphView graphview : graphviews) {
			graphview.centerGraph();
		}
	}

	public int getHighlighting() {
		return highlighting;
	}

	public void setHighlighting(int highlighting) {
		this.highlighting = highlighting;
	}

	public boolean isSynchHighlighter() {
		return synchHighlighter;
	}

	public void setSynchHighlighter(boolean synchHighlighter) {
		this.synchHighlighter = synchHighlighter;
	}

	public void setSplitter(Splitter splitter) {
		this.splitter = splitter;
		if (getGraph() != null) {
			displayGraph(getGraph());
		}
	}

	public Graph getGraph() {
		return cgraph;
	}

	public Vector<Graph> getAllGraphs() {
		return graphs;
	}

	public HashMap<String, Vector<String>> getLookUpTable() {
		return lookUpTable;
	}

	public HashMap<String, Vector<String>> getLookUpTableDouble() {
		return lookUpTableDouble;
	}

	public Vector<String> getTraits() {
		return traits;
	}

	public Vector<String> getTraitsDouble() {
		return doubleTraits;
	}

	public void setTraits(Vector<String> traits) {
		Collections.sort(traits);
		this.traits = traits;
	}

	private boolean isDouble(String trait) {
		try {
			Double d = Double.parseDouble(trait);
			return true;
		} catch (Exception ex) {
			return false;
		}

	}

	private void updateLookUpTables(Graph graph) {
		lookUpTable = new HashMap<String, Vector<String>>();
		lookUpTableDouble = new HashMap<String, Vector<String>>();
		doubleTraits.clear();
		for (String column : traits) {
			Vector<String> values = new Vector<String>();
			boolean doubles = true;
			for (Node node : graph.getAllNodes()) {
				if (node.getUserData(column) != null) {
					String trait = node.getUserData(column).toString();
					if (doubles) {
						doubles = isDouble(trait);
					}
					if (!values.contains(trait)) {
						values.add(trait);
					}
				}
			}
			if (values.size() > 0) {
				Collections.sort(values);
				if (doubles) {
					doubleTraits.add(column);
					lookUpTableDouble.put(column, values);
				}
				lookUpTable.put(column, values);
			}
		}
		Collections.sort(doubleTraits);
	}

	class CsvFilter extends FileFilter {
		public boolean accept(File f) {
			if (f.isDirectory())
				return true;
			return f.getName().toLowerCase().endsWith(".csv");
		}

		public String getDescription() {
			return "Pedigree Files (*.csv)";
		}
	}

	class JpegFilter extends FileFilter {
		public boolean accept(File f) {
			if (f.isDirectory())
				return true;
			return f.getName().toLowerCase().endsWith(".jpg");
		}

		public String getDescription() {
			return "JPEGs";
		}
	}

	class MerlinChrFilter extends FileFilter {
		public boolean accept(File f) {
			if (f.isDirectory())
				return true;
			return f.getName().toLowerCase().endsWith("chr");
		}

		public String getDescription() {
			return "Merlin Haplotypes (merlin.chr)";
		}
	}

	class MapFilter extends FileFilter {
		public boolean accept(File f) {
			if (f.isDirectory())
				return true;
			return f.getName().toLowerCase().endsWith("map");
		}

		public String getDescription() {
			return "Map File (*.map)";
		}
	}

	public DefaultNodeView getDefaultNodeView() {
		return defaultNodeView;
	}

	public void update() {
		displayGraph(cgraph);
	}

	public Haplotypes getHaplotypes() {
		return haplotypes;
	}

}
